
DELETE FROM data.changelog_entry WHERE id >= 0;
DELETE FROM data.publish_medium WHERE id >= 0;
DELETE FROM data.target_medium WHERE id >= 0;
DELETE FROM data.source_medium WHERE id >= 0;
DELETE FROM data.chapter WHERE id >= 0;
DELETE FROM data.lecture WHERE id >= 0;
DELETE FROM data.course_responsible;
DELETE FROM data."user" WHERE id >= 0;
DELETE FROM data.course WHERE id >= 0;
DELETE FROM data.featured WHERE id >= 0;
DELETE FROM data.announcement WHERE id >= 0;

SELECT setval(pg_get_serial_sequence('data.source_medium', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."source_medium"), 1), false);
SELECT setval(pg_get_serial_sequence('data.target_medium', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."target_medium"), 1), false);

CREATE OR REPLACE PROCEDURE temp_migrate_permission(old_table_name text, new_table_name text, perm_table_col text) AS $$
    DECLARE
        object_ids int[];
        obj_id int;
        object record;
        perm_ids int[];
        perm record;
        auth_type text;
        has_any_perm bool;
        public_auth bool;
        rwth_auth bool;
        fsmpi_auth bool;
        moodle_ids int[];
        passwords json;
    BEGIN
        EXECUTE 'SELECT ARRAY(SELECT id FROM ' || old_table_name || ')::int[]' INTO object_ids;

        FOREACH obj_id IN ARRAY object_ids
        LOOP
            EXECUTE 'SELECT * FROM ' || old_table_name || ' WHERE id = $1' INTO object USING obj_id;

            SELECT FALSE INTO has_any_perm;
            SELECT FALSE INTO public_auth;
            SELECT FALSE INTO rwth_auth;
            SELECT FALSE INTO fsmpi_auth;
            SELECT array[]::int[] INTO moodle_ids;
            SELECT json_build_object() INTO passwords;
            EXECUTE 'SELECT ARRAY(SELECT id FROM old_data.perm WHERE NOT deleted AND ' || perm_table_col || ' = ' || object.id::text || ')::int[]' INTO perm_ids;
            FOR perm in (SELECT * FROM old_data.perm WHERE id = ANY(perm_ids))
            LOOP
                SELECT TRUE INTO has_any_perm;
                IF perm.type = 'public' THEN
                    SELECT TRUE INTO public_auth;
                ELSIF perm.type = 'rwth' THEN
                    SELECT TRUE INTO rwth_auth;
                ELSIF perm.type = 'fsmpi' THEN
                    SELECT TRUE INTO fsmpi_auth;
                ELSIF perm.type = 'moodle' AND perm.param1 != '' THEN
                    SELECT moodle_ids || perm.param1::int INTO moodle_ids;
                ELSIF perm.type = 'password' THEN
                    SELECT (passwords::jsonb || json_build_object(perm.param1, perm.param2)::jsonb)::json INTO passwords;
                END IF;
            END LOOP;

            IF public_auth THEN
                SELECT 'public' INTO auth_type;
            ELSIF rwth_auth
                    OR fsmpi_auth
                    OR array_length(moodle_ids, 1) > 0
                    OR (SELECT COUNT(*) FROM (SELECT json_object_keys(passwords)) sub) > 0
                    THEN
                SELECT 'authentication' INTO auth_type;
            ELSIF has_any_perm THEN  -- Also for old unsupported types
                SELECT 'private' INTO auth_type;
            ELSE
                SELECT 'inherit' INTO auth_type;
            END IF;

            IF auth_type != 'authentication' THEN
                SELECT FALSE INTO rwth_auth;
                SELECT FALSE INTO fsmpi_auth;
                SELECT array[]::int[] INTO moodle_ids;
                SELECT json_build_object() INTO passwords;
            END IF;

            EXECUTE 'UPDATE ' || new_table_name || ' SET view_perm_type = $1::data.view_permissions_type, ' ||
                    ' view_perm_rwth_auth = $2, ' ||
                    ' view_perm_fsmpi_auth = $3, ' ||
                    ' view_perm_moodle_course_ids = $4, ' ||
                    ' view_perm_passwords = $5' ||
                    ' WHERE id = $6'
                    USING auth_type, rwth_auth, fsmpi_auth, moodle_ids, passwords, object.id
                    ;
        END LOOP;
    END;
$$ LANGUAGE plpgsql;


INSERT INTO data.announcement (id, deleted, visible, type, page_visibility, text, publish_time, expiration_time)
SELECT
    id,
    deleted,
    visible,
    (CASE
        WHEN level = 0 THEN 'info'
        WHEN level = 1 THEN 'info'
        WHEN level = 2 THEN 'warning'
        WHEN level = 3 THEN 'important'
    END)::data.announcement_type AS type,
    (CASE
        WHEN level = 0 THEN 'only_main_page'
        WHEN level = 1 THEN 'all_pages'
        WHEN level = 2 THEN 'all_pages'
        WHEN level = 3 THEN 'all_pages_and_embed'
    END)::data.announcement_page_visibility AS page_visibility,
    text,
    time_publish,
    time_expire
FROM old_data.announcements
;


INSERT INTO data.course (id, deleted, visible, handle, full_name, short_name, semester, organizer, topic, description,
    show_chapters_on_course, authentication_information, listed, internal_comment, allow_download, allow_embed, media_process,
    view_perm_type, view_perm_rwth_auth, view_perm_fsmpi_auth, view_perm_moodle_course_ids, view_perm_passwords,
    disable_automatic_media_process_scheduler)
SELECT
    id, deleted, visible, handle, title, short,
    CASE -- TODO better check
        WHEN semester = '' THEN 'none'
        ELSE semester
    END,
    organizer,
    subject,
    description,
    coursechapters,
    login_info,
    listed,
    internal,
    downloadable,
    NOT embedinvisible,
    json_build_object(
        'producers', json_build_array(
            json_build_object(
                'type', 'source_file',
                'tag', '',
                'output_id', 'legacy_video'
            ),
            json_build_object(
                'type', 'sample_thumbnail',
                'source_id', 'legacy_video',
                'timestamp_percent', 20,
                'output_id', 'thumbnail'
            )
        ),
        'publish_target_ids', json_build_array(
            'legacy_video', 'thumbnail'
        ),
        'publish_wait_for_full_process', True
    ),  -- TODO
    'inherit', False, False, NULL, NULL,  -- view_perm
    True  -- disable_automatic_media_process_scheduler
FROM old_data.courses_data
; -- TODO migrate legacy responsible to internal comment

CALL temp_migrate_permission('old_data.courses_data', 'data.course', 'course_id');

INSERT INTO data."user" (id, handle, name, last_login,
    enable_mail_notifications, notify_new_video, notify_edit, notify_chapter_suggested
    )
SELECT id, name, realname, last_login,
    mail_notifications, notify_new_video, notify_edit, notify_chapter_submitted
FROM old_data.users
;

INSERT INTO data.course_responsible (course_id, user_id)
SELECT DISTINCT course_id, user_id FROM old_data.responsible
WHERE course_id IN (SELECT id FROM data.course) -- There are some for unknown courses
;

INSERT INTO data.featured (id, deleted, visible, title, text, display_priority, type, image_url, course_id, lecture_id)
SELECT id, deleted, visible, title, text, 0,
    (CASE
        WHEN type = 'plain' THEN 'plain'
        WHEN type = 'image' THEN 'image'
        ELSE 'invalid' -- Will cause exception
    END)::data.featured_type,
    CASE
        WHEN type = 'image' THEN COALESCE(param, '')
        ELSE NULL
    END,
    NULL,
    NULL
FROM old_data.featured
WHERE NOT deleted
;

INSERT INTO data.lecture (id, deleted, visible, course_id, title, speaker, location, "time", duration, description,
    no_recording, livestream_planned, internal_comment,
    view_perm_type, view_perm_rwth_auth, view_perm_fsmpi_auth, view_perm_moodle_course_ids, view_perm_passwords,
    disable_automatic_media_process_scheduler)
SELECT id, deleted, visible, course_id, title, speaker, place, "time", duration, comment,
    norecording, live, internal,
    'inherit', False, False, NULL, NULL,
    False
FROM old_data.lectures_data
WHERE course_id <> 0 -- These are empty
;

CALL temp_migrate_permission('old_data.lectures_data', 'data.lecture', 'lecture_id');

INSERT INTO data.chapter (id, deleted, visible, lecture_id, start_time, name)
SELECT id, deleted, visible, lecture_id, "time", text
FROM old_data.chapters
;

DO $$
    DECLARE
        lecture record;
        video record;
        legacy_source record;
        use_custom_process bool;
        created_legacy_source_ids int[];
        thumbnail_producer_source_id text;
        custom_process_producers jsonb;
        custom_process_publish_target_ids jsonb;
        target_id int;
        source_tag text;
        process_target_id text;
    BEGIN
        FOR lecture IN (SELECT * FROM old_data.lectures_data WHERE course_id <> 0  -- course_id = 0 are empty and not migrated
        ) LOOP
            SELECT array[]::int[] INTO created_legacy_source_ids;
            SELECT NULL INTO thumbnail_producer_source_id;
            SELECT jsonb_build_array() INTO custom_process_producers;
            SELECT jsonb_build_array() INTO custom_process_publish_target_ids;
            SELECT NULL INTO target_id;
            SELECT NULL INTO source_tag;
            SELECT NULL INTO process_target_id;

            SELECT False INTO use_custom_process;
            IF (SELECT COUNT(*) FROM old_data.videos_data WHERE lecture_id = lecture.id AND NOT deleted) > 1 THEN
                SELECT TRUE INTO use_custom_process;
            ELSIF (SELECT COUNT(*) FROM old_data.videos_data WHERE lecture_id = lecture.id AND NOT deleted AND source IS NOT NULL) > 0 THEN
                SELECT TRUE INTO use_custom_process;
            END IF;

            FOR video IN (SELECT * FROM old_data.videos_data WHERE lecture_id = lecture.id AND NOT deleted) LOOP
                SELECT 'legacy_video' INTO process_target_id;
                SELECT '' INTO source_tag;
                IF use_custom_process THEN
                    SELECT 'legacy_video_' || video.id::text INTO process_target_id;
                    SELECT 'input_' || video.id::text INTO source_tag;

                    IF video.source IS NULL THEN
                        SELECT custom_process_producers || jsonb_build_object(
                            'type', 'source_file',
                            'tag', source_tag,
                            'output_id', process_target_id
                        ) INTO custom_process_producers;

                        IF thumbnail_producer_source_id IS NULL THEN
                            SELECT 'legacy_video_' || video.id::int INTO thumbnail_producer_source_id;
                        END IF;
                    ELSE
                        SELECT custom_process_producers || jsonb_build_object(
                            'type', 'downscale_video',
                            'source_id', 'legacy_source_' || video.source::text,
                            'target_vertical_resolution', CASE
                                WHEN video.video_format = 32 THEN 1440
                                WHEN video.video_format = 10 THEN 360
                                WHEN video.video_format = 5 THEN 720
                                WHEN video.video_format = 4 THEN 1080
                            END,
                            'output_id', process_target_id
                        ) INTO custom_process_producers;

                        IF thumbnail_producer_source_id IS NULL THEN
                            SELECT 'legacy_source_' || video.id::int INTO thumbnail_producer_source_id;
                        END IF;
                    END IF;
                    SELECT custom_process_publish_target_ids || jsonb_build_array(process_target_id) INTO custom_process_publish_target_ids;
                END IF;

                IF video.source IS NULL THEN
                    INSERT INTO data.source_medium (
                        file_path,
                        file_modification_time,
                        status,
                        force_immediate_sort,
                        update_time,
                        lecture_id,
                        sha256,
                        tag,
                        deleted
                    ) VALUES (
                        video.path,
                        video.time_updated,
                        'sorted'::data.source_medium_status,
                        False,
                        video.time_updated,
                        lecture.id,
                        '0000000000000000000000000000000000000000000000000000000000000000',
                        source_tag,
                        video.deleted
                    );
                ELSIF video.source NOT IN (SELECT * FROM unnest(created_legacy_source_ids)) THEN
                    SELECT * INTO legacy_source FROM old_data.sources WHERE id = video.source;

                    INSERT INTO data.source_medium (
                        file_path,
                        file_modification_time,
                        status,
                        force_immediate_sort,
                        update_time,
                        lecture_id,
                        sha256,
                        tag,
                        deleted
                    ) VALUES (
                        legacy_source.path,
                        video.time_created,
                        'sorted'::data.source_medium_status,
                        False,
                        video.time_created,
                        lecture.id,
                        '0000000000000000000000000000000000000000000000000000000000000000',
                        'legacy_source_' || legacy_source.id::text,
                        False
                    );
                    SELECT custom_process_producers || jsonb_build_object(
                        'type', 'source_file',
                        'tag', 'source_input_' || legacy_source.id::text,
                        'output_id', 'legacy_source_' || legacy_source.id::text
                    ) INTO custom_process_producers;
                    SELECT created_legacy_source_ids || legacy_source.id INTO created_legacy_source_ids;

                END IF;

                INSERT INTO data.target_medium (
                    lecture_id,
                    process_sha256,
                    process_target_id,
                    input_data_sha256,
                    is_produced,
                    to_be_replaced,
                    file_path,
                    type,
                    deleted,
                    duration_sec,
                    audio_sample_rate,
                    audio_channel_count,
                    vertical_resolution,
                    horizontal_resolution,
                    video_frame_rate_numerator,
                    video_frame_rate_denominator
                ) VALUES (
                    lecture.id,
                    '0000000000000000000000000000000000000000000000000000000000000000',
                    process_target_id,
                    '0000000000000000000000000000000000000000000000000000000000000000',
                    true,
                    false,
                    video.path,
                    'plain_video',  -- TODO
                    video.deleted,
                    -- TODO these values probably wrong
                    video.duration,
                    44000,
                    2,
                    (SELECT
                        CASE
                            WHEN resolution = '' THEN 0
                            ELSE split_part(resolution, 'x', 2)::int
                        END
                    FROM old_data.formats WHERE id = video.video_format),
                    (SELECT
                        CASE
                            WHEN resolution = '' THEN 0
                            ELSE split_part(resolution, 'x', 1)::int
                        END
                    FROM old_data.formats WHERE id = video.video_format),
                    25,
                    1
                ) RETURNING id INTO target_id;

                INSERT INTO data.publish_medium (
                    id,
                    lecture_id,
                    title,
                    target_medium_id,
                    visible,
                    deleted
                ) VALUES (
                    video.id,
                    lecture.id,
                    video.title || video.comment,  -- TODO
                    target_id,
                    video.visible,
                    video.deleted
                );
            END LOOP;

            INSERT INTO data.target_medium (
                lecture_id,
                process_sha256,
                process_target_id,
                input_data_sha256,
                is_produced,
                to_be_replaced,
                file_path,
                type,
                deleted,

                vertical_resolution,
                horizontal_resolution
            ) VALUES (
                lecture.id,
                '0000000000000000000000000000000000000000000000000000000000000000',
                'thumbnail',
                '0000000000000000000000000000000000000000000000000000000000000000',
                true,
                false,
                'thumbnail/l_' || lecture.id::text || '.jpg',
                'thumbnail',
                False,
                -- TODO these values probably wrong
                640,
                640
            ) RETURNING id INTO target_id;

            INSERT INTO data.publish_medium (
                id,
                lecture_id,
                title,
                target_medium_id,
                visible,
                deleted
            ) VALUES (
                lecture.id + 100000,
                lecture.id,
                '',
                target_id,
                True,
                False
            );

            IF use_custom_process THEN
                SELECT custom_process_producers || jsonb_build_object(
                    'type', 'sample_thumbnail',
                    'source_id', thumbnail_producer_source_id,
                    'timestamp_percent', 20,
                    'output_id', process_target_id
                ) INTO custom_process_producers;
                SELECT custom_process_publish_target_ids || jsonb_build_array('thumbnail') INTO custom_process_publish_target_ids;

                UPDATE data.lecture as lecture_table SET media_process = json_build_object(
                    'producers', custom_process_producers::json,
                    'publish_target_ids', custom_process_publish_target_ids::json,
                    'publish_wait_for_full_process', True
                ) WHERE id = lecture.id;
            END IF;


        END LOOP;
    END;
$$;

SELECT setval(pg_get_serial_sequence('data.announcement', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."announcement"), 1), false);
SELECT setval(pg_get_serial_sequence('data.course', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."course"), 1), false);
SELECT setval(pg_get_serial_sequence('data.featured', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."featured"), 1), false);
SELECT setval(pg_get_serial_sequence('data.lecture', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."lecture"), 1), false);
SELECT setval(pg_get_serial_sequence('data.chapter', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."chapter"), 1), false);
SELECT setval(pg_get_serial_sequence('data.source_medium', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."source_medium"), 1), false);
SELECT setval(pg_get_serial_sequence('data.target_medium', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."target_medium"), 1), false);
SELECT setval(pg_get_serial_sequence('data.publish_medium', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."publish_medium"), 1), false);
SELECT setval(pg_get_serial_sequence('data.user', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."user"), 1), false);
SELECT setval(pg_get_serial_sequence('data.changelog_entry', 'id'), COALESCE((SELECT MAX("id")+1 FROM "data"."changelog_entry"), 1), false);