import sys
from datetime import datetime
from typing import Literal

from videoag_common.database import *

import api
from api.miscellaneous import DIAGNOSTICS_TRACKER, DiagnosticsCounter

DbValueType = int | str | bytes | datetime

_TransactionType = Literal["read", "write"]


def _create_diagnostics_counters(id: str) -> dict[_TransactionType, DiagnosticsCounter]:
    return {
        "read": DIAGNOSTICS_TRACKER.register_counter(f"database.transaction.read.{id}"),
        "write": DIAGNOSTICS_TRACKER.register_counter(f"database.transaction.write.{id}")
    }


_COUNTERS_TRANSACTION_COUNT = _create_diagnostics_counters("count")
_COUNTERS_TRANSACTION_ABORTED_BY_USER = _create_diagnostics_counters("aborted_by_user")
_COUNTERS_TRANSACTION_CONFLICTS = _create_diagnostics_counters("conflicts")
_COUNTERS_TRANSACTION_ABORTS_AFTER_REPEATED_CONFLICTS = _create_diagnostics_counters("aborts_after_repeated_conflicts")


class ApiDatabase(Database):

    def _on_transaction_started(self, writeable: bool):
        _COUNTERS_TRANSACTION_COUNT["write" if writeable else "read"].trigger()
    
    def _on_transaction_aborted_by_user(self, writeable: bool):
        _COUNTERS_TRANSACTION_ABORTED_BY_USER["write" if writeable else "read"].trigger()
    
    def _on_transaction_conflict(self, writeable: bool):
        _COUNTERS_TRANSACTION_CONFLICTS["write" if writeable else "read"].trigger()
    
    def _on_transaction_aborted_after_repeated_conflict(self, writeable: bool):
        _COUNTERS_TRANSACTION_ABORTS_AFTER_REPEATED_CONFLICTS["write" if writeable else "read"].trigger()


database = ApiDatabase(api.config["DATABASE"])
