import random
import sched
import threading
import time
import traceback
from time import sleep

import api

__SCHEDULER = sched.scheduler()

_DISABLE_SCHEDULER = api.config.get("DISABLE_SCHEDULER", False)


def scheduled_function(delay_sec: int, initial_delay_sec: int or None = None, priority: int = 0):
    if initial_delay_sec is None:
        initial_delay_sec = random.randint(1, 120)
    
    def decorator(func):
        def execute_scheduled():
            print(f"Scheduler: Executing {func.__name__}")
            start_time = time.time_ns()
            try:
                func()
            except Exception as e:
                print(f"Scheduler: An exception occurred during execution of {func.__name__}:")
                traceback.print_exception(e)
                # TODO notify
            
            total_time = time.time_ns() - start_time
            print(f"Scheduler: Execution of {func.__name__} took {(total_time//1000)/1000}ms")
            __SCHEDULER.enter(delay_sec, priority, execute_scheduled)
        
        __SCHEDULER.enter(initial_delay_sec, priority, execute_scheduled)
        print(f"Scheduler: Registered '{func.__name__}' to be executed every {delay_sec}s (first execution in {initial_delay_sec}s) {'(Scheduler is disabled)' if _DISABLE_SCHEDULER else ''}")
        return func
    
    return decorator


def __run_scheduler_thread():
    while True:
        __SCHEDULER.run()
        sleep(5)


if not _DISABLE_SCHEDULER:
    threading.Thread(target=__run_scheduler_thread, daemon=True).start()
