from flask import request
import re

from api.routes import *


FEEDBACK_EMAIL_MAX_LENGTH = 128
FEEDBACK_EMAIL_REGEX = "\\S+@\\S+\\.\\S+"
FEEDBACK_EMAIL_PATTERN = re.compile(FEEDBACK_EMAIL_REGEX)
FEEDBACK_TEXT_MAX_LENGTH = 16384

FEEDBACK_MIN_PAGE_SIZE = 10
FEEDBACK_MAX_PAGE_SIZE = 500


@api_route("/feedback/new", "PUT",
           request_objects=[
               ("email", "?string", "May not be longer than 128 characters. Must match `\\S+@\\S+\\.\\S+` (More sophisticated "
                                    "checks are allowed, though)"),
               ("text", "string", "May not be longer than 16384 characters")
           ],
           response_description="`201 CREATED` if the feedback was saved. Otherwise, an [api error](#api_error).")
def api_route_feedback_new():
    json_request = get_client_json(request)
    email = json_request.get_string("email", FEEDBACK_EMAIL_MAX_LENGTH, optional=True)
    text = json_request.get_string("text", FEEDBACK_TEXT_MAX_LENGTH)
    if email is not None and not FEEDBACK_EMAIL_PATTERN.fullmatch(email):
        raise json_request.get("email").raise_error(f"Does not match pattern {FEEDBACK_EMAIL_REGEX}")
    
    return {}, HTTP_501_NOT_IMPLEMENTED  # TODO send email
