from flask import redirect

from videoag_common.objects import *
from api.authentication import is_authenticated
from api.routes import *


@api_route("/resources/target_medium/<int:target_medium_id>", "GET", allow_while_readonly=True,
           no_documentation=True)
def api_route_access_target_medium(target_medium_id: int):
    is_mod = is_moderator()
    medium = database.query_one_or_none_and_expunge(TargetMedium.select(
        is_mod,
        [
            LOAD_TARGET_MEDIUM_PUBLISH_MEDIUM,
            LOAD_TARGET_MEDIUM_LECTURE,
            LOAD_PUBLISH_MEDIUM_LECTURE,
            LOAD_LECTURE_COURSE
        ],
        visibility_check=False,
    ).where(TargetMedium.id == target_medium_id))
    if medium is None:
        raise ApiClientException(ERROR_UNKNOWN_OBJECT)
    if not medium.has_access(is_mod=is_mod):
        raise ApiClientException(ERROR_UNAUTHORIZED)
    
    lecture = medium.publish_medium.lecture
    assert isinstance(lecture, Lecture)
    if not is_authenticated(lecture.effective_view_permissions):
        raise ApiClientException(ERROR_UNAUTHORIZED)
    
    # TODO
    
    if isinstance(medium, ThumbnailTargetMedium):
        return redirect("https://video.fsmpi.rwth-aachen.de/files/thumbnail/l_17253.jpg")
    else:
        return redirect("https://video.fsmpi.rwth-aachen.de/files/pub/23ws-buk/23ws-buk-231012-1080p.mp4")
