from api.miscellaneous import *
from videoag_common.objects import User


class ApiSession:
    
    def __init__(self):
        super().__init__()
        from flask import session
        self.session = session
    
    @property
    def permanent(self) -> bool:
        return self.session.permanent
    
    @permanent.setter
    def permanent(self, val: bool):
        self.session.permanent = val
    
    @property
    def is_rwth_authenticated(self) -> bool:
        return self.session.get("is_rwth_authenticated", False) is True
    
    @is_rwth_authenticated.setter
    def is_rwth_authenticated(self, val: bool):
        self.session["is_rwth_authenticated"] = val
    
    @property
    def is_fsmpi_authenticated(self) -> bool:
        return self.session.get("is_fsmpi_authenticated", False) is True
    
    @is_fsmpi_authenticated.setter
    def is_fsmpi_authenticated(self, val: bool):
        self.session["is_fsmpi_authenticated"] = val
    
    def is_in_moodle_course(self, course_id: int) -> bool:
        return course_id in self.session.get("moodle_course_ids", [])
    
    def is_in_any_moodle_course(self) -> bool:
        return len(self.session.get("moodle_course_ids", [])) > 0
    
    def set_moodle_course_ids(self, course_ids: list[int]):
        self.session["moodle_course_ids"] = course_ids
        self.session.modified = True
    
    def is_password_authenticated(self, username: str, password: str) -> bool:
        hashes = self.session.get("authenticated_password_hashes", [])
        return hash_sha256(f"{username}:{password}") in hashes
    
    def has_any_password_authenticated(self):
        return len(self.session.get("authenticated_password_hashes", [])) > 0
    
    def add_authenticated_password(self, username: str, password: str):
        self.session.setdefault("authenticated_password_hashes", []).append(
            hash_sha256(f"{username}:{password}")
        )
        self.session.modified = True
    
    @property
    def running_oauth_scope(self) -> str or None:
        return self.session.get("running_oauth_scope", None)
    
    @running_oauth_scope.setter
    def running_oauth_scope(self, val: str or None):
        if val is None:
            self.session.pop("running_oauth_scope", None)
        else:
            self.session["running_oauth_scope"] = val
    
    @property
    def running_oauth_code(self) -> str or None:
        return self.session.get("running_oauth_code", None)
    
    @running_oauth_code.setter
    def running_oauth_code(self, val: str or None):
        if val is None:
            self.session.pop("running_oauth_code", None)
        else:
            self.session["running_oauth_code"] = val
    
    @property
    def csrf_token(self) -> str or None:
        return self.session.get("csrf_token", None)
    
    @csrf_token.setter
    def csrf_token(self, val: str or None):
        if val is None:
            self.session.pop("csrf_token", None)
        else:
            self.session["csrf_token"] = val
    
    def get_user_id(self) -> int or None:
        if "user" not in self.session:
            return None
        return self.session["user"]["id"]
    
    def get_user_info(self) -> dict or None:
        if "user" not in self.session:
            return None
        return self.session["user"]
    
    def set_user(self, user: User or None):
        if user is None:
            self.session.pop("user", None)
            return
        self.session["user"] = {
            "id": user.id,
            "handle": user.handle,
            "name": user.name,
        }


api_session = ApiSession()
