from videoag_common.miscellaneous.constants import *

from api_test import ApiTest
from videoag_common.test.object_data import *
import videoag_common.test.object_data as object_data


class CoursesTest(ApiTest):
    
    def test_get_all_courses(self):
        self.do_json_request(
            "GET",
            "/courses",
            None,
            {
                "courses": {
                    "2": TEST_DATA_COURSE_2_NO_LEC,
                    "3": TEST_DATA_COURSE_3_NO_LEC
                }
            },
            ["default_authentication_methods"],
        )
        self.do_json_request(
            "GET",
            "/courses",
            None,
            {
                "courses": {
                    "2": TEST_DATA_COURSE_2_NO_LEC_MOD,
                    "3": TEST_DATA_COURSE_3_NO_LEC_MOD,
                    "13": TEST_DATA_COURSE_13_NO_LEC_MOD,
                    "62": TEST_DATA_COURSE_62_NO_LEC_MOD,
                }
            },
            ["default_authentication_methods"],
            use_moderator_login=True
        )
    
    def test_get_course(self):
        self.do_json_request(
            "GET",
            "/course/1",  # Deleted
            None,
            expected_response_code=HTTP_404_NOT_FOUND
        )
        self.do_json_request(
            "GET",
            "/course/1",  # Deleted
            None,
            expected_response_code=HTTP_404_NOT_FOUND,
            use_moderator_login=True
        )
        for id, vis in [(2, True), (3, True), (13, False), (62, True)]:
            print(f"Testing {id}")
            if vis:
                self.do_json_request(
                    "GET",
                    f"/course/{id}",
                    None,
                    getattr(object_data, f"TEST_DATA_COURSE_{id}_NO_LEC"),
                    ["default_authentication_methods"]
                )
                self.do_json_request(
                    "GET",
                    f"/course/{id}?include_lectures=false",
                    None,
                    getattr(object_data, f"TEST_DATA_COURSE_{id}_NO_LEC"),
                    ["default_authentication_methods"]
                )
                self.do_json_request(
                    "GET",
                    f"/course/{id}?include_lectures=true",
                    None,
                    getattr(object_data, f"TEST_DATA_COURSE_{id}"),
                    ["default_authentication_methods", "publish_media"]
                )
            else:
                self.do_json_request(
                    "GET",
                    f"/course/{id}",
                    None,
                    expected_response_code=HTTP_401_UNAUTHORIZED
                )
                self.do_json_request(
                    "GET",
                    f"/course/{id}?include_lectures=true",
                    None,
                    expected_response_code=HTTP_401_UNAUTHORIZED
                )
            self.do_json_request(
                "GET",
                f"/course/{id}",
                None,
                getattr(object_data, f"TEST_DATA_COURSE_{id}_NO_LEC_MOD"),
                ["default_authentication_methods"],
                use_moderator_login=True
            )
            self.do_json_request(
                "GET",
                f"/course/{id}?include_lectures=false",
                None,
                getattr(object_data, f"TEST_DATA_COURSE_{id}_NO_LEC_MOD"),
                ["default_authentication_methods"],
                use_moderator_login=True
            )
            self.do_json_request(
                "GET",
                f"/course/{id}?include_lectures=true",
                None,
                getattr(object_data, f"TEST_DATA_COURSE_{id}_MOD"),
                ["default_authentication_methods", "publish_media"],
                use_moderator_login=True
            )
            
    def test_get_lecture(self):
        self.do_json_request(
            "GET",
            "/lecture/187",  # Deleted
            None,
            expected_response_code=HTTP_404_NOT_FOUND
        )
        self.do_json_request(
            "GET",
            "/course/187",  # Deleted
            None,
            expected_response_code=HTTP_404_NOT_FOUND,
            use_moderator_login=True
        )
        for id in [1, 2, 3, 25, 26, 29, 185, 186, 1186, 1187, 1188]:
            print(f"Testing {id}")
            mod_data = getattr(object_data, f"TEST_DATA_LECTURE_{id}_MOD")
            if (mod_data["visible"]
                    and getattr(object_data, f"TEST_DATA_COURSE_{mod_data['course_id']}_MOD")["visible"]):
                self.do_json_request(
                    "GET",
                    f"/lecture/{id}",
                    None,
                    {
                        "course": getattr(object_data, f"TEST_DATA_COURSE_{mod_data['course_id']}_NO_LEC"),
                        "lecture": getattr(object_data, f"TEST_DATA_LECTURE_{id}")
                    },
                    ["default_authentication_methods", "publish_media"]
                )
            else:
                self.do_json_request(
                    "GET",
                    f"/lecture/{id}",
                    None,
                    expected_response_code=HTTP_401_UNAUTHORIZED
                )
            self.do_json_request(
                "GET",
                f"/lecture/{id}",
                None,
                {
                    "course": getattr(object_data, f"TEST_DATA_COURSE_{mod_data['course_id']}_NO_LEC_MOD"),
                    "lecture": getattr(object_data, f"TEST_DATA_LECTURE_{id}_MOD")
                },
                ["default_authentication_methods", "publish_media"],
                use_moderator_login=True
            )
    
    def test_search(self):
        self.do_json_request(
            "GET",
            "/search?q=LO" + "O" * 2000 + "ONG",
            None,
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "GET",
            "/search?q=Many" + " Words" * 20,
            None,
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "GET",
            "/search",
            None,
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "GET",
            "/search?q= ",
            None,
            expected_response_code=HTTP_400_BAD_REQUEST
        )
        self.do_json_request(
            "GET",
            "/search?q=Graphen Komplexität Finanzierung Grundlagen",
            None,
            {
                "courses": [2],
                "lectures": [
                    TEST_DATA_LECTURE_25,
                ],
                "course_context": {
                    "2": TEST_DATA_COURSE_2_NO_LEC,
                    "3": TEST_DATA_COURSE_3_NO_LEC
                }
            },
            ignore_array_order_keys=["default_authentication_methods", "authentication_methods", "publish_media"]
        )
        self.do_json_request(
            "GET",
            "/search?q=Graphen Komplexität Finanzierung Grundlagen",
            None,
            {
                "courses": [2, 62],
                "lectures": [
                    TEST_DATA_LECTURE_25_MOD,
                    TEST_DATA_LECTURE_1186_MOD,
                ],
                "course_context": {
                    "2": TEST_DATA_COURSE_2_NO_LEC_MOD,
                    "3": TEST_DATA_COURSE_3_NO_LEC_MOD,
                    "62": TEST_DATA_COURSE_62_NO_LEC_MOD,
                }
            },
            ignore_array_order_keys=["default_authentication_methods", "authentication_methods", "publish_media"],
            use_moderator_login=True
        )
        self.do_json_request(
            "GET",
            "/search?q=Graphen",
            None,
            {
                "courses": [],
                "lectures": [
                    TEST_DATA_LECTURE_25
                ],
                "course_context": {
                    "3": TEST_DATA_COURSE_3_NO_LEC
                }
            },
            ignore_array_order_keys=["default_authentication_methods", "authentication_methods", "publish_media"]
        )
        self.do_json_request(
            "GET",
            "/search?q=Graphen",
            None,
            {
                "courses": [],
                "lectures": [
                    TEST_DATA_LECTURE_25_MOD
                ],
                "course_context": {
                    "3": TEST_DATA_COURSE_3_NO_LEC_MOD
                }
            },
            ignore_array_order_keys=["default_authentication_methods", "authentication_methods", "publish_media"],
            use_moderator_login=True
        )
